/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.role.ui.android.spi;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import android.content.Context;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import it.tidalwave.role.ui.android.ViewFactory;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import lombok.RequiredArgsConstructor;
import static it.tidalwave.util.thread.ThreadType.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor
public class InflatingViewFactory implements ViewFactory
  {
    private final int resourceId;
    
    @CheckForNull
    private Context previousContext;
    
    @CheckForNull
    private LayoutInflater layoutInflater;
    
    @Nonnull
    public View createView (final @Nonnull Context context, final @Nonnull ViewGroup viewGroup)
      {
        return getLayoutInflater(context).inflate(resourceId, viewGroup);        
      }
    
    @Nonnull @ThreadConfined(type=UI)
    private LayoutInflater getLayoutInflater (final @Nonnull Context context)
      {
        if ((layoutInflater == null) || (previousContext != context))
          {
            layoutInflater = LayoutInflater.from(context);
            previousContext = context;
          }
        
        return layoutInflater;
      }
  }
