/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.ui.android.widget;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.List;
import it.tidalwave.util.As;
import it.tidalwave.util.thread.annotation.ThreadConfined;
import android.widget.BaseAdapter;
import android.widget.Filter;
import static it.tidalwave.util.thread.ThreadType.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class AsWidgetFilter extends Filter
  {
    @Nonnull
    protected final List<? extends As> objects;

    @Nonnull
    private final BaseAdapter adapter;

    @Nonnull
    private int[] indexTable;

    @Nonnegative
    private int filteredCount;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @ThreadConfined(type=UI) // seems an Android requirement for creating a Filter
    public AsWidgetFilter (final @Nonnull List<? extends As> taxa,
                           final @Nonnull BaseAdapter adapter)
      {
        this.objects = taxa;
        this.adapter = adapter;
        this.filteredCount = taxa.size();
        this.indexTable = new int[taxa.size()];

        for (int i = 0; i < indexTable.length; i++)
          {
            this.indexTable[i] = i;
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnegative
    public int getCount()
      {
        return filteredCount;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public As getItem (final @Nonnegative int index)
      {
        return objects.get(indexTable[index]);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull @ThreadConfined(type=NOT_UI)
    protected FilterResults performFiltering (final @Nonnull CharSequence match)
      {
        // FIXME: can be improved:
        // first, create a stack of pairs (match, FilterResults); in this way, when you delete the last character, you
        // have the previous result immediately
        // second, when you add a new character, you can filter only on the previous results
        final FilterResults results = new FilterResults();
        final int[] newIndexes = new int[objects.size()];
        results.values = newIndexes;
        results.count = 0;

        for (int i = 0; i < objects.size(); i++)
          {
            if (filter(objects.get(i), match))
              {
                newIndexes[results.count++] = i;
              }
          }

        return results;
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull @ThreadConfined(type=NOT_UI)
    protected abstract boolean filter (@Nonnull As object, @Nonnull CharSequence match);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void publishResults (final @Nonnull CharSequence match,
                                   final @Nonnull FilterResults filterResults)
      {
        indexTable = (int[])filterResults.values;
        filteredCount = filterResults.count;
        adapter.notifyDataSetChanged();
      }
  }
